import { Card } from '@/components/ui/card';
import { formatPrice, formatPercentage, cn } from '@/lib/utils';
import { useMarketDataUpdates, usePriceUpdates } from '@/hooks/useWebSocket';
import { useQuery } from '@tanstack/react-query';
import { useEffect, useState } from 'react';
import type { MarketData } from '@shared/schema';

interface CurrencyPairsProps {
  selectedPair: string;
  onPairSelect: (pair: string) => void;
}

export function CurrencyPairs({ selectedPair, onPairSelect }: CurrencyPairsProps) {
  const marketDataUpdates = useMarketDataUpdates();
  const priceUpdates = usePriceUpdates();
  const [liveData, setLiveData] = useState<Record<string, MarketData>>({});

  const { data: initialMarketData } = useQuery({
    queryKey: ['/api/market-data'],
    refetchInterval: 10000,
  });

  // Initialize live data with API data
  useEffect(() => {
    if (initialMarketData) {
      const dataMap = initialMarketData.reduce((acc: Record<string, MarketData>, item: MarketData) => {
        acc[item.pair] = item;
        return acc;
      }, {});
      setLiveData(dataMap);
    }
  }, [initialMarketData]);

  // Update with WebSocket market data
  useEffect(() => {
    if (marketDataUpdates.length > 0) {
      const dataMap = marketDataUpdates.reduce((acc: Record<string, MarketData>, item) => {
        acc[item.pair] = item;
        return acc;
      }, {});
      setLiveData(prev => ({ ...prev, ...dataMap }));
    }
  }, [marketDataUpdates]);

  // Update with real-time price updates
  useEffect(() => {
    if (priceUpdates.length > 0) {
      setLiveData(prev => {
        const updated = { ...prev };
        priceUpdates.forEach(update => {
          if (updated[update.pair]) {
            const lastPrice = updated[update.pair].price;
            const change = update.price - lastPrice;
            const changePercent = (change / lastPrice) * 100;
            
            updated[update.pair] = {
              ...updated[update.pair],
              price: update.price,
              change,
              changePercent,
              timestamp: new Date(),
            };
          }
        });
        return updated;
      });
    }
  }, [priceUpdates]);

  const pairs = Object.values(liveData);
  
  const getTrendColor = (changePercent: number) => {
    if (changePercent > 0) return 'border-trading-success';
    if (changePercent < 0) return 'border-trading-danger';
    return 'border-trading-muted';
  };

  const getTextColor = (changePercent: number) => {
    if (changePercent > 0) return 'text-trading-success';
    if (changePercent < 0) return 'text-trading-danger';
    return 'text-trading-muted';
  };

  return (
    <Card className="bg-trading-gray rounded-xl p-6">
      <h3 className="text-lg font-semibold mb-4">Active Pairs</h3>
      
      {pairs.length === 0 ? (
        <div className="text-center text-trading-muted py-8">
          <div className="text-4xl mb-2">📊</div>
          <p>Loading market data...</p>
        </div>
      ) : (
        <div className="grid grid-cols-6 gap-3">
          {pairs.map((pair) => (
            <div
              key={pair.pair}
              onClick={() => onPairSelect(pair.pair)}
              className={cn(
                'bg-trading-light hover:bg-trading-slate rounded-lg p-3 cursor-pointer transition border-l-2',
                getTrendColor(pair.changePercent),
                selectedPair === pair.pair && 'ring-2 ring-trading-warning'
              )}
            >
              <div className="font-semibold text-sm mb-1">{pair.pair}</div>
              <div className={cn('font-mono text-xs', getTextColor(pair.changePercent))}>
                {formatPrice(pair.price, pair.pair)}
              </div>
              <div className={cn('font-mono text-xs', getTextColor(pair.changePercent))}>
                {formatPercentage(pair.changePercent)}
              </div>
            </div>
          ))}
        </div>
      )}
    </Card>
  );
}
