import { Card } from '@/components/ui/card';
import { Badge } from '@/components/ui/badge';
import { Progress } from '@/components/ui/progress';
import { TrendingUp, TrendingDown, Activity, Clock, Target, AlertTriangle } from 'lucide-react';
import { useQuery } from '@tanstack/react-query';
import { useMarketDataUpdates, usePriceUpdates } from '@/hooks/useWebSocket';
import { useState, useEffect } from 'react';
import { formatPrice, formatPercentage } from '@/lib/utils';
import type { MarketData } from '@shared/schema';

interface MarketAnalysisProps {
  selectedPair: string;
}

export function MarketAnalysis({ selectedPair }: MarketAnalysisProps) {
  const [trendStrength, setTrendStrength] = useState(0);
  const [volatility, setVolatility] = useState('Normal');
  const [marketSentiment, setMarketSentiment] = useState<'Bullish' | 'Bearish' | 'Neutral'>('Neutral');
  const [priceHistory, setPriceHistory] = useState<number[]>([]);
  
  const marketDataUpdates = useMarketDataUpdates();
  const priceUpdates = usePriceUpdates();

  const { data: candleData } = useQuery({
    queryKey: ['/api/candle-data', selectedPair],
    refetchInterval: 30000,
  });

  // Calculate market metrics
  useEffect(() => {
    if (candleData && candleData.length >= 10) {
      const closes = candleData.map((c: any) => c.close);
      const recentPrices = closes.slice(-10);
      
      // Calculate trend strength (0-100)
      const priceChanges = recentPrices.slice(1).map((price: number, i: number) => 
        ((price - recentPrices[i]) / recentPrices[i]) * 100
      );
      
      const avgChange = priceChanges.reduce((sum: number, change: number) => sum + change, 0) / priceChanges.length;
      const strength = Math.min(Math.abs(avgChange) * 20, 100);
      setTrendStrength(Math.round(strength));
      
      // Calculate volatility
      const volatilityValue = Math.sqrt(
        priceChanges.reduce((sum: number, change: number) => sum + Math.pow(change, 2), 0) / priceChanges.length
      );
      
      if (volatilityValue > 0.15) setVolatility('High');
      else if (volatilityValue > 0.08) setVolatility('Medium');
      else setVolatility('Low');
      
      // Market sentiment
      if (avgChange > 0.05) setMarketSentiment('Bullish');
      else if (avgChange < -0.05) setMarketSentiment('Bearish');
      else setMarketSentiment('Neutral');
      
      setPriceHistory(recentPrices);
    }
  }, [candleData]);

  const getSentimentColor = () => {
    switch (marketSentiment) {
      case 'Bullish': return 'text-trading-success';
      case 'Bearish': return 'text-trading-danger';
      default: return 'text-trading-muted';
    }
  };

  const getVolatilityColor = () => {
    switch (volatility) {
      case 'High': return 'text-trading-danger';
      case 'Medium': return 'text-trading-warning';
      default: return 'text-trading-success';
    }
  };

  const getTradingRecommendation = () => {
    if (trendStrength > 70 && volatility !== 'High') {
      return {
        action: 'STRONG SIGNAL',
        message: `High probability ${marketSentiment.toLowerCase()} trend`,
        color: 'text-trading-success',
        icon: Target
      };
    } else if (trendStrength > 50) {
      return {
        action: 'MODERATE SIGNAL',
        message: `Moderate ${marketSentiment.toLowerCase()} momentum`,
        color: 'text-trading-warning',
        icon: Activity
      };
    } else if (volatility === 'High') {
      return {
        action: 'CAUTION',
        message: 'High volatility - trade carefully',
        color: 'text-trading-danger',
        icon: AlertTriangle
      };
    } else {
      return {
        action: 'WAIT',
        message: 'Weak signals - wait for better setup',
        color: 'text-trading-muted',
        icon: Clock
      };
    }
  };

  const recommendation = getTradingRecommendation();

  return (
    <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-4">
      {/* Market Sentiment */}
      <Card className="bg-trading-gray p-4">
        <div className="flex items-center justify-between mb-3">
          <span className="text-sm text-trading-muted">Market Sentiment</span>
          {marketSentiment === 'Bullish' ? (
            <TrendingUp className="w-4 h-4 text-trading-success" />
          ) : marketSentiment === 'Bearish' ? (
            <TrendingDown className="w-4 h-4 text-trading-danger" />
          ) : (
            <Activity className="w-4 h-4 text-trading-muted" />
          )}
        </div>
        <div className={`text-lg font-bold ${getSentimentColor()}`}>
          {marketSentiment}
        </div>
        <div className="text-xs text-trading-muted mt-1">
          Based on recent price action
        </div>
      </Card>

      {/* Trend Strength */}
      <Card className="bg-trading-gray p-4">
        <div className="flex items-center justify-between mb-3">
          <span className="text-sm text-trading-muted">Trend Strength</span>
          <div className="text-sm font-mono font-bold text-trading-warning">
            {trendStrength}%
          </div>
        </div>
        <Progress value={trendStrength} className="h-2 mb-2" />
        <div className="text-xs text-trading-muted">
          {trendStrength > 70 ? 'Very Strong' : 
           trendStrength > 50 ? 'Strong' : 
           trendStrength > 30 ? 'Moderate' : 'Weak'}
        </div>
      </Card>

      {/* Volatility */}
      <Card className="bg-trading-gray p-4">
        <div className="flex items-center justify-between mb-3">
          <span className="text-sm text-trading-muted">Volatility</span>
          <Activity className={`w-4 h-4 ${getVolatilityColor()}`} />
        </div>
        <div className={`text-lg font-bold ${getVolatilityColor()}`}>
          {volatility}
        </div>
        <div className="text-xs text-trading-muted mt-1">
          Market movement intensity
        </div>
      </Card>

      {/* Trading Recommendation */}
      <Card className="bg-trading-gray p-4 md:col-span-2 lg:col-span-3">
        <div className="flex items-center space-x-3">
          <recommendation.icon className={`w-6 h-6 ${recommendation.color}`} />
          <div className="flex-1">
            <div className={`font-bold text-lg ${recommendation.color}`}>
              {recommendation.action}
            </div>
            <div className="text-sm text-trading-muted">
              {recommendation.message}
            </div>
          </div>
          <Badge 
            className={`${
              recommendation.action === 'STRONG SIGNAL' ? 'bg-trading-success' :
              recommendation.action === 'MODERATE SIGNAL' ? 'bg-trading-warning' :
              recommendation.action === 'CAUTION' ? 'bg-trading-danger' :
              'bg-trading-slate'
            } text-white font-semibold px-4 py-2`}
          >
            {selectedPair}
          </Badge>
        </div>
      </Card>

      {/* Price Momentum */}
      <Card className="bg-trading-gray p-4 md:col-span-2 lg:col-span-2">
        <h4 className="font-semibold mb-3">Price Momentum (Last 10 Candles)</h4>
        <div className="grid grid-cols-5 gap-2">
          {priceHistory.slice(-10).map((price, index, array) => {
            const prevPrice = array[index - 1];
            const isUp = index === 0 || price > prevPrice;
            const change = index === 0 ? 0 : ((price - prevPrice) / prevPrice) * 100;
            
            return (
              <div key={index} className="text-center">
                <div className={`w-8 h-8 rounded-full flex items-center justify-center text-xs font-bold mx-auto mb-1 ${
                  isUp ? 'bg-trading-success text-trading-dark' : 'bg-trading-danger text-white'
                }`}>
                  {isUp ? '↑' : '↓'}
                </div>
                <div className="text-xs font-mono text-trading-muted">
                  {Math.abs(change).toFixed(1)}%
                </div>
              </div>
            );
          })}
        </div>
      </Card>

      {/* Support/Resistance Levels */}
      <Card className="bg-trading-gray p-4">
        <h4 className="font-semibold mb-3">Key Levels</h4>
        {candleData && candleData.length > 0 && (
          <div className="space-y-2">
            <div className="flex justify-between">
              <span className="text-sm text-trading-muted">Resistance</span>
              <span className="font-mono text-sm text-trading-danger">
                {formatPrice(Math.max(...candleData.slice(-20).map((c: any) => c.high)), selectedPair)}
              </span>
            </div>
            <div className="flex justify-between">
              <span className="text-sm text-trading-muted">Current</span>
              <span className="font-mono text-sm font-bold text-trading-warning">
                {formatPrice(candleData[candleData.length - 1]?.close || 0, selectedPair)}
              </span>
            </div>
            <div className="flex justify-between">
              <span className="text-sm text-trading-muted">Support</span>
              <span className="font-mono text-sm text-trading-success">
                {formatPrice(Math.min(...candleData.slice(-20).map((c: any) => c.low)), selectedPair)}
              </span>
            </div>
          </div>
        )}
      </Card>
    </div>
  );
}