export interface IndicatorConfig {
  period: number;
  source: 'close' | 'open' | 'high' | 'low';
}

export function calculateSMA(data: number[], period: number): number[] {
  const result: number[] = [];
  
  for (let i = period - 1; i < data.length; i++) {
    const sum = data.slice(i - period + 1, i + 1).reduce((acc, val) => acc + val, 0);
    result.push(sum / period);
  }
  
  return result;
}

export function calculateEMA(data: number[], period: number): number[] {
  const result: number[] = [];
  const multiplier = 2 / (period + 1);
  
  if (data.length === 0) return result;
  
  // Start with SMA for the first value
  const firstSMA = data.slice(0, period).reduce((acc, val) => acc + val, 0) / period;
  result.push(firstSMA);
  
  // Calculate EMA for the rest
  for (let i = period; i < data.length; i++) {
    const ema = (data[i] * multiplier) + (result[result.length - 1] * (1 - multiplier));
    result.push(ema);
  }
  
  return result;
}

export function calculateRSI(data: number[], period: number = 14): number[] {
  const result: number[] = [];
  
  if (data.length < period + 1) return result;
  
  let gains: number[] = [];
  let losses: number[] = [];
  
  // Calculate initial gains and losses
  for (let i = 1; i <= period; i++) {
    const change = data[i] - data[i - 1];
    gains.push(change > 0 ? change : 0);
    losses.push(change < 0 ? Math.abs(change) : 0);
  }
  
  let avgGain = gains.reduce((acc, val) => acc + val, 0) / period;
  let avgLoss = losses.reduce((acc, val) => acc + val, 0) / period;
  
  const calculateRSIValue = (avgGain: number, avgLoss: number): number => {
    if (avgLoss === 0) return 100;
    const rs = avgGain / avgLoss;
    return 100 - (100 / (1 + rs));
  };
  
  result.push(calculateRSIValue(avgGain, avgLoss));
  
  // Calculate RSI for remaining data points
  for (let i = period + 1; i < data.length; i++) {
    const change = data[i] - data[i - 1];
    const gain = change > 0 ? change : 0;
    const loss = change < 0 ? Math.abs(change) : 0;
    
    avgGain = (avgGain * (period - 1) + gain) / period;
    avgLoss = (avgLoss * (period - 1) + loss) / period;
    
    result.push(calculateRSIValue(avgGain, avgLoss));
  }
  
  return result;
}

export interface BollingerBands {
  upper: number[];
  middle: number[];
  lower: number[];
}

export function calculateBollingerBands(data: number[], period: number = 20, stdDev: number = 2): BollingerBands {
  const sma = calculateSMA(data, period);
  const upper: number[] = [];
  const lower: number[] = [];
  
  for (let i = 0; i < sma.length; i++) {
    const dataIndex = i + period - 1;
    const slice = data.slice(dataIndex - period + 1, dataIndex + 1);
    
    const variance = slice.reduce((acc, val) => acc + Math.pow(val - sma[i], 2), 0) / period;
    const standardDeviation = Math.sqrt(variance);
    
    upper.push(sma[i] + (standardDeviation * stdDev));
    lower.push(sma[i] - (standardDeviation * stdDev));
  }
  
  return {
    upper,
    middle: sma,
    lower
  };
}

export function calculateMACD(data: number[], fastPeriod: number = 12, slowPeriod: number = 26, signalPeriod: number = 9) {
  const fastEMA = calculateEMA(data, fastPeriod);
  const slowEMA = calculateEMA(data, slowPeriod);
  
  const macdLine: number[] = [];
  const startIndex = slowPeriod - fastPeriod;
  
  for (let i = 0; i < slowEMA.length; i++) {
    macdLine.push(fastEMA[i + startIndex] - slowEMA[i]);
  }
  
  const signalLine = calculateEMA(macdLine, signalPeriod);
  const histogram: number[] = [];
  const histogramStartIndex = signalPeriod - 1;
  
  for (let i = histogramStartIndex; i < macdLine.length; i++) {
    histogram.push(macdLine[i] - signalLine[i - histogramStartIndex]);
  }
  
  return {
    macd: macdLine,
    signal: signalLine,
    histogram
  };
}

export function findSupportResistance(highs: number[], lows: number[], lookback: number = 20): { support: number; resistance: number } {
  if (highs.length < lookback || lows.length < lookback) {
    return {
      support: Math.min(...lows),
      resistance: Math.max(...highs)
    };
  }
  
  const recentHighs = highs.slice(-lookback);
  const recentLows = lows.slice(-lookback);
  
  // Find local minima and maxima
  const supportLevels: number[] = [];
  const resistanceLevels: number[] = [];
  
  for (let i = 1; i < recentLows.length - 1; i++) {
    if (recentLows[i] < recentLows[i - 1] && recentLows[i] < recentLows[i + 1]) {
      supportLevels.push(recentLows[i]);
    }
  }
  
  for (let i = 1; i < recentHighs.length - 1; i++) {
    if (recentHighs[i] > recentHighs[i - 1] && recentHighs[i] > recentHighs[i + 1]) {
      resistanceLevels.push(recentHighs[i]);
    }
  }
  
  return {
    support: supportLevels.length > 0 ? Math.min(...supportLevels) : Math.min(...recentLows),
    resistance: resistanceLevels.length > 0 ? Math.max(...resistanceLevels) : Math.max(...recentHighs)
  };
}

export function detectCandlestickPatterns(candles: { open: number; high: number; low: number; close: number }[]): string[] {
  const patterns: string[] = [];
  
  if (candles.length < 2) return patterns;
  
  const current = candles[candles.length - 1];
  const previous = candles[candles.length - 2];
  
  const currentBody = Math.abs(current.close - current.open);
  const previousBody = Math.abs(previous.close - previous.open);
  
  const currentRange = current.high - current.low;
  const previousRange = previous.high - previous.low;
  
  // Doji pattern
  if (currentBody < currentRange * 0.1) {
    patterns.push('Doji');
  }
  
  // Pin Bar / Hammer
  const upperShadow = current.high - Math.max(current.open, current.close);
  const lowerShadow = Math.min(current.open, current.close) - current.low;
  
  if (lowerShadow > currentBody * 2 && upperShadow < currentBody) {
    patterns.push('Pin Bar');
  }
  
  // Engulfing patterns
  if (previous.close < previous.open && current.close > current.open) {
    if (current.open < previous.close && current.close > previous.open) {
      patterns.push('Bullish Engulfing');
    }
  }
  
  if (previous.close > previous.open && current.close < current.open) {
    if (current.open > previous.close && current.close < previous.open) {
      patterns.push('Bearish Engulfing');
    }
  }
  
  return patterns;
}
