import { type ClassValue, clsx } from "clsx"
import { twMerge } from "tailwind-merge"

export function cn(...inputs: ClassValue[]) {
  return twMerge(clsx(inputs))
}

export function formatPrice(price: number, pair: string): string {
  if (pair.includes('JPY')) {
    return price.toFixed(3);
  } else if (pair.includes('BTC') || pair.includes('OTC')) {
    return price.toLocaleString('en-US', { maximumFractionDigits: 0 });
  } else {
    return price.toFixed(5);
  }
}

export function formatPriceChange(change: number, pair: string): string {
  const formattedChange = Math.abs(change);
  let result = '';
  
  if (pair.includes('JPY')) {
    result = formattedChange.toFixed(3);
  } else if (pair.includes('BTC') || pair.includes('OTC')) {
    result = formattedChange.toLocaleString('en-US', { maximumFractionDigits: 0 });
  } else {
    result = formattedChange.toFixed(5);
  }
  
  return change >= 0 ? `+${result}` : `-${result}`;
}

export function formatPercentage(percentage: number): string {
  return `${percentage >= 0 ? '+' : ''}${percentage.toFixed(2)}%`;
}

export function formatTime(date: Date): string {
  return date.toISOString().substr(11, 8) + ' UTC';
}

export function formatTimeShort(date: Date): string {
  return date.toISOString().substr(11, 5);
}

export function getNextCandleTime(): Date {
  const now = new Date();
  const nextMinute = new Date(now.getTime() + 60000);
  nextMinute.setSeconds(0, 0);
  return nextMinute;
}

export function getSecondsUntilNextCandle(): number {
  const now = new Date();
  return 60 - now.getSeconds();
}

export function formatCountdown(seconds: number): string {
  const minutes = Math.floor(seconds / 60);
  const remainingSeconds = seconds % 60;
  return `${minutes.toString().padStart(2, '0')}:${remainingSeconds.toString().padStart(2, '0')}`;
}

export function getSignalColor(direction: 'CALL' | 'PUT'): string {
  return direction === 'CALL' ? 'text-trading-success' : 'text-trading-danger';
}

export function getSignalBgColor(direction: 'CALL' | 'PUT'): string {
  return direction === 'CALL' ? 'bg-trading-success' : 'bg-trading-danger';
}

export function getIndicatorStatusColor(status: 'active' | 'warning' | 'inactive'): string {
  switch (status) {
    case 'active':
      return 'bg-trading-success';
    case 'warning':
      return 'bg-trading-warning';
    case 'inactive':
      return 'bg-trading-slate';
    default:
      return 'bg-trading-slate';
  }
}

export function getSignalConfidenceColor(confidence: number): string {
  if (confidence >= 90) return 'text-trading-success';
  if (confidence >= 80) return 'text-trading-warning';
  return 'text-trading-muted';
}

export function debounce<T extends (...args: any[]) => any>(
  func: T,
  wait: number
): (...args: Parameters<T>) => void {
  let timeout: NodeJS.Timeout;
  
  return (...args: Parameters<T>) => {
    clearTimeout(timeout);
    timeout = setTimeout(() => func(...args), wait);
  };
}

export function isMarketOpen(): boolean {
  // Simplified market hours check (assuming 24/5 for forex)
  const now = new Date();
  const day = now.getUTCDay();
  const hour = now.getUTCHours();
  
  // Closed on weekends (Saturday 22:00 UTC to Monday 22:00 UTC)
  if (day === 0 || (day === 6 && hour >= 22)) {
    return false;
  }
  if (day === 1 && hour < 22) {
    return false;
  }
  
  return true;
}
