import { useState, useEffect } from 'react';
import { Card } from '@/components/ui/card';
import { Badge } from '@/components/ui/badge';
import { Progress } from '@/components/ui/progress';
import { 
  BarChart3, 
  Clock, 
  Activity, 
  Settings, 
  History,
  Target,
  Wifi,
  WifiOff
} from 'lucide-react';
import { TradingChart } from '@/components/TradingChart';
import { LiveSignals } from '@/components/LiveSignals';
import { MarketAnalysis } from '@/components/MarketAnalysis';
import { CurrencyPairs } from '@/components/CurrencyPairs';
import { SignalHistory } from '@/components/SignalHistory';
import { TradeEntryGuide } from '@/components/TradeEntryGuide';
import { formatTime, formatCountdown, getSecondsUntilNextCandle, getNextCandleTime } from '@/lib/utils';
import { useMarketStatus, useTimeUpdates } from '@/hooks/useWebSocket';
import { useQuery } from '@tanstack/react-query';
import type { StrategyPerformance } from '@shared/schema';

export default function Dashboard() {
  const [selectedPair, setSelectedPair] = useState('EUR/USD');
  const [countdown, setCountdown] = useState(0);
  const [nextCandleTime, setNextCandleTime] = useState<Date>(new Date());
  
  const { status: marketStatus, isConnected } = useMarketStatus();
  const currentTime = useTimeUpdates();

  const { data: strategyPerformance } = useQuery({
    queryKey: ['/api/strategy-performance'],
    refetchInterval: 60000,
  });

  // Update countdown timer
  useEffect(() => {
    const updateCountdown = () => {
      const seconds = getSecondsUntilNextCandle();
      setCountdown(seconds);
      setNextCandleTime(getNextCandleTime());
    };

    updateCountdown();
    const interval = setInterval(updateCountdown, 1000);
    
    return () => clearInterval(interval);
  }, []);

  const getStatusColor = () => {
    if (!isConnected) return 'bg-trading-danger';
    if (marketStatus === 'ACTIVE') return 'bg-trading-success pulse-green';
    return 'bg-trading-warning';
  };

  const getStatusText = () => {
    if (!isConnected) return 'DISCONNECTED';
    return marketStatus;
  };

  return (
    <div className="flex h-screen overflow-hidden bg-trading-dark text-white">
      {/* Sidebar Navigation */}
      <div className="w-64 bg-trading-gray border-r border-trading-light flex-shrink-0">
        <div className="p-6">
          <div className="flex items-center space-x-3 mb-8">
            <div className="w-10 h-10 bg-trading-warning rounded-lg flex items-center justify-center">
              <BarChart3 className="text-trading-dark text-xl" />
            </div>
            <div>
              <h1 className="text-xl font-bold">Quotex Engine</h1>
              <p className="text-trading-muted text-sm">Signal Pro v2.1</p>
            </div>
          </div>
          
          {/* Navigation Menu */}
          <nav className="space-y-2">
            <a href="/" className="flex items-center space-x-3 px-4 py-3 bg-trading-light rounded-lg text-trading-warning">
              <Activity className="w-5 h-5" />
              <span>Dashboard</span>
            </a>
            <a href="/live-charts" className="flex items-center space-x-3 px-4 py-3 hover:bg-trading-light rounded-lg text-trading-muted hover:text-white transition">
              <BarChart3 className="w-5 h-5" />
              <span>Live Charts</span>
            </a>
            <a href="/signal-history" className="flex items-center space-x-3 px-4 py-3 hover:bg-trading-light rounded-lg text-trading-muted hover:text-white transition">
              <History className="w-5 h-5" />
              <span>Signal History</span>
            </a>
            <a href="/settings" className="flex items-center space-x-3 px-4 py-3 hover:bg-trading-light rounded-lg text-trading-muted hover:text-white transition">
              <Settings className="w-5 h-5" />
              <span>Settings</span>
            </a>
          </nav>
          
          {/* Market Status */}
          <Card className="mt-8 p-4 bg-trading-light border-trading-slate">
            <div className="flex items-center justify-between mb-2">
              <span className="text-sm text-trading-muted">Market Status</span>
              <div className={`w-2 h-2 rounded-full ${getStatusColor()}`}></div>
            </div>
            <div className="text-sm font-semibold text-trading-success">{getStatusText()}</div>
            <div className="text-xs text-trading-muted mt-1 flex items-center">
              {isConnected ? (
                <Wifi className="w-3 h-3 mr-1" />
              ) : (
                <WifiOff className="w-3 h-3 mr-1" />
              )}
              {formatTime(currentTime)}
            </div>
          </Card>
        </div>
      </div>

      {/* Main Content Area */}
      <div className="flex-1 flex flex-col overflow-hidden">
        {/* Top Header */}
        <div className="bg-trading-gray border-b border-trading-light p-4">
          <div className="flex items-center justify-between">
            <div className="flex items-center space-x-6">
              <h2 className="text-2xl font-bold">Live Trading Dashboard</h2>
              <Badge className="bg-trading-light text-trading-success border-trading-success">
                <div className="w-2 h-2 bg-trading-success rounded-full animate-pulse mr-2"></div>
                Real-time Data
              </Badge>
            </div>
            <div className="flex items-center space-x-4">
              {/* Next Signal Countdown */}
              <div className="bg-trading-warning text-trading-dark px-4 py-2 rounded-lg font-mono font-semibold">
                <Clock className="inline w-4 h-4 mr-2" />
                {formatCountdown(countdown)}
              </div>
              <div className="text-right">
                <div className="text-sm text-trading-muted">Next Candle</div>
                <div className="font-mono font-semibold">
                  {nextCandleTime.toISOString().substr(11, 5)}:00
                </div>
              </div>
            </div>
          </div>
        </div>

        {/* Dashboard Content */}
        <div className="flex-1 p-6 overflow-auto">
          <div className="grid grid-cols-12 gap-6 h-full">
            
            {/* Left Panel - Live Signals & Strategy Performance */}
            <div className="col-span-4 space-y-6">
              <TradeEntryGuide />
              <LiveSignals />
              
              {/* Strategy Performance */}
              <Card className="bg-trading-gray rounded-xl p-6">
                <h3 className="text-lg font-semibold mb-4">Strategy Performance</h3>
                <div className="space-y-4">
                  {strategyPerformance?.map((strategy) => (
                    <div key={strategy.name} className="flex items-center justify-between">
                      <span className="text-sm">{strategy.name}</span>
                      <div className="flex items-center space-x-2">
                        <Progress 
                          value={strategy.accuracy} 
                          className="w-24 h-2"
                        />
                        <span className="font-mono text-sm text-trading-success">
                          {strategy.accuracy}%
                        </span>
                      </div>
                    </div>
                  )) || (
                    <div className="text-center text-trading-muted py-4">
                      <Target className="w-8 h-8 mx-auto mb-2 opacity-50" />
                      <p>Loading performance data...</p>
                    </div>
                  )}
                </div>
              </Card>
              
              {/* Martingale Tracker */}
              <Card className="bg-trading-gray rounded-xl p-6">
                <h3 className="text-lg font-semibold mb-4">Martingale Tracker</h3>
                <div className="space-y-3">
                  <div className="flex items-center justify-between">
                    <span className="text-sm">Current Step</span>
                    <span className="font-mono text-trading-warning">Step 1</span>
                  </div>
                  <div className="flex items-center justify-between">
                    <span className="text-sm">Next Stake</span>
                    <span className="font-mono text-white">$10.00</span>
                  </div>
                  <div className="flex items-center justify-between">
                    <span className="text-sm">Recovery Target</span>
                    <span className="font-mono text-trading-success">$8.50</span>
                  </div>
                  <Card className="bg-trading-light p-3 mt-4">
                    <div className="text-xs text-trading-muted mb-1">Recovery Progress</div>
                    <Progress value={0} className="w-full h-2" />
                  </Card>
                </div>
              </Card>
            </div>
            
            {/* Center Panel - Charts & Indicators */}
            <div className="col-span-8 space-y-6">
              {/* Main Chart */}
              <TradingChart 
                selectedPair={selectedPair}
                onPairChange={setSelectedPair}
              />
              
              {/* Market Analysis Panel */}
              <MarketAnalysis selectedPair={selectedPair} />
              
              {/* Currency Pairs Grid */}
              <CurrencyPairs 
                selectedPair={selectedPair}
                onPairSelect={setSelectedPair}
              />
              
              {/* Recent Signals History */}
              <SignalHistory />
            </div>
          </div>
        </div>
      </div>
    </div>
  );
}
