import { storage } from './storage';
import type { LiveSignal, TechnicalIndicator, CandleData, StrategyPerformance } from '@shared/schema';

interface TechnicalIndicators {
  ema5: number;
  ema20: number;
  rsi: number;
  bollingerUpper: number;
  bollingerLower: number;
  bollingerMiddle: number;
  support: number;
  resistance: number;
}

export class SignalEngine {
  private candleData: Map<string, CandleData[]> = new Map();
  private readonly pairs = ['EUR/USD', 'GBP/USD', 'USD/JPY', 'AUD/USD', 'USD/CAD', 'GBP/JPY', 'BTC/USD OTC'];
  private readonly strategies = ['EMA Crossover', 'Price Action', 'RSI + S/R', 'Bollinger Bands'];
  
  constructor() {
    this.initializeCandleData();
  }

  private initializeCandleData() {
    this.pairs.forEach(pair => {
      this.candleData.set(pair, this.generateInitialCandles(pair, 100));
    });
  }

  private generateInitialCandles(pair: string, count: number): CandleData[] {
    const candles: CandleData[] = [];
    let basePrice = this.getBasePriceForPair(pair);
    const now = new Date();

    for (let i = count - 1; i >= 0; i--) {
      const timestamp = new Date(now.getTime() - i * 60000);
      const volatility = this.getVolatilityForPair(pair);
      
      const open = basePrice;
      const change = (Math.random() - 0.5) * volatility;
      const close = open + change;
      const high = Math.max(open, close) + Math.random() * (volatility / 4);
      const low = Math.min(open, close) - Math.random() * (volatility / 4);
      
      candles.push({
        timestamp,
        open: parseFloat(open.toFixed(5)),
        high: parseFloat(high.toFixed(5)),
        low: parseFloat(low.toFixed(5)),
        close: parseFloat(close.toFixed(5)),
      });
      
      basePrice = close;
    }
    
    return candles;
  }

  private getBasePriceForPair(pair: string): number {
    const basePrices: { [key: string]: number } = {
      'EUR/USD': 1.08945,
      'GBP/USD': 1.24782,
      'USD/JPY': 148.234,
      'AUD/USD': 0.67123,
      'USD/CAD': 1.36789,
      'GBP/JPY': 184.567,
      'BTC/USD OTC': 43245,
    };
    return basePrices[pair] || 1.0;
  }

  private getVolatilityForPair(pair: string): number {
    const volatilities: { [key: string]: number } = {
      'EUR/USD': 0.001,
      'GBP/USD': 0.0015,
      'USD/JPY': 0.5,
      'AUD/USD': 0.0012,
      'USD/CAD': 0.0008,
      'GBP/JPY': 0.8,
      'BTC/USD OTC': 200,
    };
    return volatilities[pair] || 0.001;
  }

  public addNewCandle(pair: string): CandleData {
    const candles = this.candleData.get(pair) || [];
    const lastCandle = candles[candles.length - 1];
    
    if (!lastCandle) {
      throw new Error(`No candle data for pair ${pair}`);
    }

    const volatility = this.getVolatilityForPair(pair);
    const open = lastCandle.close;
    const change = (Math.random() - 0.5) * volatility;
    const close = open + change;
    const high = Math.max(open, close) + Math.random() * (volatility / 4);
    const low = Math.min(open, close) - Math.random() * (volatility / 4);
    
    const newCandle: CandleData = {
      timestamp: new Date(),
      open: parseFloat(open.toFixed(5)),
      high: parseFloat(high.toFixed(5)),
      low: parseFloat(low.toFixed(5)),
      close: parseFloat(close.toFixed(5)),
    };

    candles.push(newCandle);
    
    // Keep only last 200 candles
    if (candles.length > 200) {
      candles.shift();
    }
    
    this.candleData.set(pair, candles);
    
    // Update market data
    const lastPrice = candles[candles.length - 2]?.close || open;
    const priceChange = close - lastPrice;
    const changePercent = (priceChange / lastPrice) * 100;
    
    storage.updateMarketData({
      pair,
      price: close,
      change: priceChange,
      changePercent,
    });

    return newCandle;
  }

  private calculateTechnicalIndicators(pair: string): TechnicalIndicators {
    const candles = this.candleData.get(pair) || [];
    const closes = candles.map(c => c.close);
    const highs = candles.map(c => c.high);
    const lows = candles.map(c => c.low);

    return {
      ema5: this.calculateEMA(closes, 5),
      ema20: this.calculateEMA(closes, 20),
      rsi: this.calculateRSI(closes, 14),
      bollingerUpper: this.calculateBollingerBands(closes, 20).upper,
      bollingerLower: this.calculateBollingerBands(closes, 20).lower,
      bollingerMiddle: this.calculateBollingerBands(closes, 20).middle,
      support: this.calculateSupport(lows),
      resistance: this.calculateResistance(highs),
    };
  }

  private calculateEMA(prices: number[], period: number): number {
    if (prices.length < period) return prices[prices.length - 1] || 0;
    
    const multiplier = 2 / (period + 1);
    let ema = prices.slice(0, period).reduce((sum, price) => sum + price, 0) / period;
    
    for (let i = period; i < prices.length; i++) {
      ema = (prices[i] * multiplier) + (ema * (1 - multiplier));
    }
    
    return ema;
  }

  private calculateRSI(prices: number[], period: number): number {
    if (prices.length < period + 1) return 50;
    
    let gains = 0;
    let losses = 0;
    
    for (let i = 1; i <= period; i++) {
      const change = prices[i] - prices[i - 1];
      if (change > 0) gains += change;
      else losses -= change;
    }
    
    let avgGain = gains / period;
    let avgLoss = losses / period;
    
    for (let i = period + 1; i < prices.length; i++) {
      const change = prices[i] - prices[i - 1];
      if (change > 0) {
        avgGain = (avgGain * (period - 1) + change) / period;
        avgLoss = (avgLoss * (period - 1)) / period;
      } else {
        avgGain = (avgGain * (period - 1)) / period;
        avgLoss = (avgLoss * (period - 1) - change) / period;
      }
    }
    
    const rs = avgGain / avgLoss;
    return 100 - (100 / (1 + rs));
  }

  private calculateBollingerBands(prices: number[], period: number): { upper: number; middle: number; lower: number } {
    if (prices.length < period) {
      const lastPrice = prices[prices.length - 1] || 0;
      return { upper: lastPrice, middle: lastPrice, lower: lastPrice };
    }
    
    const recentPrices = prices.slice(-period);
    const average = recentPrices.reduce((sum, price) => sum + price, 0) / period;
    const variance = recentPrices.reduce((sum, price) => sum + Math.pow(price - average, 2), 0) / period;
    const stdDev = Math.sqrt(variance);
    
    return {
      upper: average + (stdDev * 2),
      middle: average,
      lower: average - (stdDev * 2),
    };
  }

  private calculateSupport(lows: number[]): number {
    if (lows.length < 20) return Math.min(...lows);
    return Math.min(...lows.slice(-20));
  }

  private calculateResistance(highs: number[]): number {
    if (highs.length < 20) return Math.max(...highs);
    return Math.max(...highs.slice(-20));
  }

  public generateSignal(pair: string): LiveSignal | null {
    const indicators = this.calculateTechnicalIndicators(pair);
    const candles = this.candleData.get(pair) || [];
    
    if (candles.length < 5) return null; // Need sufficient data
    
    const currentPrice = candles[candles.length - 1]?.close || 0;
    const previousCandle = candles[candles.length - 2];
    const priceChange = currentPrice - previousCandle.close;
    const priceChangePercent = (priceChange / previousCandle.close) * 100;
    
    const signals: { direction: 'CALL' | 'PUT'; strategy: string; confidence: number; weight: number }[] = [];
    
    // Enhanced EMA Crossover Strategy with momentum
    const emaDiff = indicators.ema5 - indicators.ema20;
    const emaPercentDiff = (emaDiff / indicators.ema20) * 100;
    
    if (emaDiff > 0 && Math.abs(emaPercentDiff) > 0.01) {
      const confidence = Math.min(88 + Math.abs(emaPercentDiff) * 100, 95);
      signals.push({ direction: 'CALL', strategy: 'EMA Crossover', confidence, weight: 3 });
    } else if (emaDiff < 0 && Math.abs(emaPercentDiff) > 0.01) {
      const confidence = Math.min(88 + Math.abs(emaPercentDiff) * 100, 95);
      signals.push({ direction: 'PUT', strategy: 'EMA Crossover', confidence, weight: 3 });
    }
    
    // Enhanced RSI Strategy with divergence
    if (indicators.rsi < 25 && currentPrice > indicators.support * 1.0005) {
      const oversoldStrength = (30 - indicators.rsi) / 30;
      const confidence = Math.min(82 + oversoldStrength * 12, 94);
      signals.push({ direction: 'CALL', strategy: 'RSI Oversold', confidence, weight: 2 });
    } else if (indicators.rsi > 75 && currentPrice < indicators.resistance * 0.9995) {
      const overboughtStrength = (indicators.rsi - 70) / 30;
      const confidence = Math.min(82 + overboughtStrength * 12, 94);
      signals.push({ direction: 'PUT', strategy: 'RSI Overbought', confidence, weight: 2 });
    }
    
    // Enhanced Bollinger Bands with volatility
    const bbWidth = (indicators.bollingerUpper - indicators.bollingerLower) / indicators.bollingerMiddle;
    if (currentPrice <= indicators.bollingerLower && bbWidth > 0.02) {
      const penetration = (indicators.bollingerLower - currentPrice) / indicators.bollingerLower;
      const confidence = Math.min(78 + penetration * 200, 92);
      signals.push({ direction: 'CALL', strategy: 'Bollinger Squeeze', confidence, weight: 2 });
    } else if (currentPrice >= indicators.bollingerUpper && bbWidth > 0.02) {
      const penetration = (currentPrice - indicators.bollingerUpper) / indicators.bollingerUpper;
      const confidence = Math.min(78 + penetration * 200, 92);
      signals.push({ direction: 'PUT', strategy: 'Bollinger Squeeze', confidence, weight: 2 });
    }
    
    // Enhanced Support/Resistance with price action
    const supportDistance = Math.abs(currentPrice - indicators.support) / currentPrice;
    const resistanceDistance = Math.abs(currentPrice - indicators.resistance) / currentPrice;
    
    if (supportDistance < 0.0008 && priceChangePercent > 0) {
      const bounceStrength = Math.abs(priceChangePercent) * 10;
      const confidence = Math.min(75 + bounceStrength, 89);
      signals.push({ direction: 'CALL', strategy: 'Support Bounce', confidence, weight: 2 });
    } else if (resistanceDistance < 0.0008 && priceChangePercent < 0) {
      const rejectionStrength = Math.abs(priceChangePercent) * 10;
      const confidence = Math.min(75 + rejectionStrength, 89);
      signals.push({ direction: 'PUT', strategy: 'Resistance Reject', confidence, weight: 2 });
    }
    
    // Price Action Pattern Recognition
    const candlePatterns = this.detectCandlestickPatterns(candles.slice(-3));
    if (candlePatterns.length > 0) {
      for (const pattern of candlePatterns) {
        if (pattern === 'Bullish Engulfing' && indicators.rsi < 60) {
          signals.push({ direction: 'CALL', strategy: 'Bullish Engulfing', confidence: 86, weight: 2 });
        } else if (pattern === 'Bearish Engulfing' && indicators.rsi > 40) {
          signals.push({ direction: 'PUT', strategy: 'Bearish Engulfing', confidence: 86, weight: 2 });
        } else if (pattern === 'Pin Bar') {
          const direction = currentPrice > previousCandle.close ? 'CALL' : 'PUT';
          signals.push({ direction, strategy: 'Pin Bar', confidence: 82, weight: 1 });
        }
      }
    }
    
    // Require at least 3 confirming signals with minimum total weight
    if (signals.length < 3) return null;
    
    // Separate by direction and calculate weighted consensus
    const callSignals = signals.filter(s => s.direction === 'CALL');
    const putSignals = signals.filter(s => s.direction === 'PUT');
    
    const callWeight = callSignals.reduce((sum, s) => sum + s.weight, 0);
    const putWeight = putSignals.reduce((sum, s) => sum + s.weight, 0);
    
    // Need strong consensus (at least 6 total weight)
    if (Math.max(callWeight, putWeight) < 6) return null;
    
    const isCall = callWeight > putWeight;
    const relevantSignals = isCall ? callSignals : putSignals;
    
    // Calculate weighted average confidence
    const totalWeight = relevantSignals.reduce((sum, s) => sum + s.weight, 0);
    const weightedConfidence = relevantSignals.reduce((sum, s) => sum + (s.confidence * s.weight), 0) / totalWeight;
    
    // Add consensus bonus (more strategies = higher confidence)
    const consensusBonus = Math.min(relevantSignals.length * 2, 8);
    const finalConfidence = Math.min(weightedConfidence + consensusBonus, 97);
    
    // Only return signals with confidence >= 85%
    if (finalConfidence < 85) return null;
    
    return {
      pair,
      direction: isCall ? 'CALL' : 'PUT',
      confidence: Math.round(finalConfidence),
      strategies: relevantSignals.map(s => s.strategy),
      timestamp: new Date(),
    };
  }
  
  private detectCandlestickPatterns(candles: CandleData[]): string[] {
    const patterns: string[] = [];
    
    if (candles.length < 2) return patterns;
    
    const current = candles[candles.length - 1];
    const previous = candles[candles.length - 2];
    
    const currentBody = Math.abs(current.close - current.open);
    const previousBody = Math.abs(previous.close - previous.open);
    
    const currentRange = current.high - current.low;
    const previousRange = previous.high - previous.low;
    
    // Doji pattern - small body relative to range
    if (currentBody < currentRange * 0.1) {
      patterns.push('Doji');
    }
    
    // Pin Bar / Hammer - long wick, small body
    const upperShadow = current.high - Math.max(current.open, current.close);
    const lowerShadow = Math.min(current.open, current.close) - current.low;
    
    if (lowerShadow > currentBody * 2.5 && upperShadow < currentBody * 0.5) {
      patterns.push('Pin Bar');
    }
    
    // Engulfing patterns - current candle completely engulfs previous
    if (previous.close < previous.open && current.close > current.open) {
      if (current.open <= previous.close && current.close >= previous.open && currentBody > previousBody * 1.2) {
        patterns.push('Bullish Engulfing');
      }
    }
    
    if (previous.close > previous.open && current.close < current.open) {
      if (current.open >= previous.close && current.close <= previous.open && currentBody > previousBody * 1.2) {
        patterns.push('Bearish Engulfing');
      }
    }
    
    return patterns;
  }

  public getTechnicalIndicators(pair: string): TechnicalIndicator[] {
    const indicators = this.calculateTechnicalIndicators(pair);
    const candles = this.candleData.get(pair) || [];
    const currentPrice = candles[candles.length - 1]?.close || 0;
    
    return [
      {
        name: 'EMA 5/20',
        value: indicators.ema5 > indicators.ema20 ? 'BULLISH' : 'BEARISH',
        signal: indicators.ema5 > indicators.ema20 ? 'BULLISH' : 'BEARISH',
        status: 'active',
      },
      {
        name: 'RSI (14)',
        value: Math.round(indicators.rsi * 10) / 10,
        signal: indicators.rsi < 30 ? 'BULLISH' : indicators.rsi > 70 ? 'BEARISH' : 'NEUTRAL',
        status: indicators.rsi < 30 || indicators.rsi > 70 ? 'warning' : 'active',
      },
      {
        name: 'Bollinger',
        value: currentPrice <= indicators.bollingerLower ? 'BOUNCE' : currentPrice >= indicators.bollingerUpper ? 'BOUNCE' : 'MIDDLE',
        signal: currentPrice <= indicators.bollingerLower ? 'BULLISH' : currentPrice >= indicators.bollingerUpper ? 'BEARISH' : 'NEUTRAL',
        status: currentPrice <= indicators.bollingerLower || currentPrice >= indicators.bollingerUpper ? 'active' : 'inactive',
      },
      {
        name: 'S/R Levels',
        value: Math.abs(currentPrice - indicators.support) < Math.abs(currentPrice - indicators.resistance) ? 'SUPPORT' : 'RESISTANCE',
        signal: Math.abs(currentPrice - indicators.support) < Math.abs(currentPrice - indicators.resistance) ? 'BULLISH' : 'BEARISH',
        status: 'active',
      },
    ];
  }

  public getCandleData(pair: string): CandleData[] {
    return this.candleData.get(pair) || [];
  }

  public getAllPairs(): string[] {
    return this.pairs;
  }

  public getStrategyPerformance(): StrategyPerformance[] {
    // Mock performance data for now
    return [
      { name: 'EMA Crossover', accuracy: 94, signals: 45, wins: 42, losses: 3 },
      { name: 'Price Action', accuracy: 89, signals: 38, wins: 34, losses: 4 },
      { name: 'RSI + S/R', accuracy: 91, signals: 41, wins: 37, losses: 4 },
      { name: 'Bollinger Bands', accuracy: 86, signals: 33, wins: 28, losses: 5 },
    ];
  }
}

export const signalEngine = new SignalEngine();
